@@ -43,14 +43,19 @@ class PetabImporter():
             petab_problem: petab_MS.Problem):
         self.petab_problem = petab_problem
 
-    def create_model(self):
+    def create_model(self, executable=None):
         """Create a model.
+        Parameters
 
+        ----------
+        executable:
+            path to Morpheus executable.
         Returns
         -------
-        Morpheus_model: A model of a class MorpheusModels
+        Morpheus_model:
+            A model of a class MorpheusModels
         """
-        return create_model(self)
+        return create_model(self, executable)
 
     def create_prior(self) -> pyabc.Distribution:
         """Create prior.
@@ -165,13 +170,13 @@ class PetabImporter():
 
         return get_exp_cond_map(condition_df=self.petab_problem.condition_df)
 
-    def import_petab_problem(self):
-        """
-        Import model from PEtab problem.
-        :return:
-            The imported model.
-        """
-        return import_petab_problem(self.petab_problem)
+    # def import_petab_problem(self):
+    #     """
+    #     Import model from PEtab problem.
+    #     :return:
+    #         The imported model.
+    #     """
+    #     return import_petab_problem(self.petab_problem)
 
     def _sanity_check(self):
         """Some checks to identify potential problems."""
@@ -184,32 +189,86 @@ class PetabImporter():
                 "in this case and just performs sampling on the prior scale.")
 
 
-def create_model(importer):
+def create_model(importer, executable):
+    """
+    Import model from petab importer.
+
+    Parameters
+    ----------
+    importer:
+        A petab problem importer.
+    executable:
+        path to Morpheus executable.
+
+    Returns
+    -------
+    The imported model.
+    """
+    if 'par_map' not in importer.petab_problem.parameter_df:
+        raise ValueError("'par_map' is missing in the parameter table")
+
     conditions = importer.petab_problem.condition_df
     cond_status = get_nr_exp_condition(conditions)
     par_map_imported = importer.get_par_map()
     par_scale = importer.petab_problem.get_optimization_parameter_scales()
+    if 'observableFormula' in importer.petab_problem.observable_df:
+        ss_post_process_callable = get_ss_post_process(
+            importer.petab_problem.observable_df)
 
     if cond_status == "no_condition":
-        model_con1 = MorpheusModel(
-            importer.petab_problem.model_file, par_map=par_map_imported,
-            par_scale=par_scale, timeout=100)
+        if executable is not None:
+            model_con1 = MorpheusModel(
+                importer.petab_problem.model_file,
+                par_map=par_map_imported,
+                par_scale=par_scale,
+                ss_post_processing=ss_post_process_callable,
+                executable=executable)
+        else:
+            model_con1 = MorpheusModel(
+                importer.petab_problem.model_file,
+                par_map=par_map_imported,
+                par_scale=par_scale,
+                ss_post_processing=ss_post_process_callable)
         return model_con1
     elif cond_status == "one_condition":
         exp_cond_imported = importer.get_exp_cond_map()
-        model_con1 = MorpheusModel(
-            importer.petab_problem.model_file, par_map=par_map_imported,
-            exp_cond_map={'condition1': exp_cond_imported['condition1']},
-            par_scale=par_scale, timeout=100)
+        if executable is not None:
+            model_con1 = MorpheusModel(
+                importer.petab_problem.model_file,
+                par_map=par_map_imported,
+                exp_cond_map={'condition1': exp_cond_imported['condition1']},
+                par_scale=par_scale,
+                ss_post_processing=ss_post_process_callable,
+                executable=executable)
+        else:
+            model_con1 = MorpheusModel(
+                importer.petab_problem.model_file,
+                par_map=par_map_imported,
+                exp_cond_map={'condition1': exp_cond_imported['condition1']},
+                par_scale=par_scale,
+                ss_post_processing=ss_post_process_callable)
         return model_con1
     else:
         exp_cond_imported = importer.get_exp_cond_map()
         models_list = []
-        for cond_key, cond_val in exp_cond_imported.items():
-            tmp_model = MorpheusModel(
-                importer.petab_problem.model_file, par_map=par_map_imported,
-                exp_cond_map={cond_key: cond_val},
-                par_scale=par_scale, timeout=100)
+        if executable is not None:
+            for cond_key, cond_val in exp_cond_imported.items():
+                tmp_model = MorpheusModel(
+                    importer.petab_problem.model_file,
+                    par_map=par_map_imported,
+                    exp_cond_map={cond_key: cond_val},
+                    par_scale=par_scale,
+                    ss_post_processing=ss_post_process_callable,
+                    executable=executable)
+        else:
+            for cond_key, cond_val in exp_cond_imported.items():
+                tmp_model = MorpheusModel(
+                    importer.petab_problem.model_file,
+                    par_map=par_map_imported,
+                    exp_cond_map={cond_key: cond_val},
+                    par_scale=par_scale,
+                    ss_post_processing=ss_post_process_callable)
+
             models_list.append(tmp_model)
 
         return MorpheusModels(models_list=models_list)
@@ -383,53 +442,49 @@ def get_ss_post_process(observable_df: pd.DataFrame) -> Union[Callable, dict]:
     return post_processing_ss_dict
 
 
-def import_petab_problem(
-        petab_problem: petab_MS.Problem,
-        model_dir) -> 'MorpheusModel':
-    """
-    Import model from petab problem.
-
-    :param petab_problem:
-        A petab problem containing all relevant information on the model.
-
-    :param model_dir:
-        Directory to Morpheus model.
-
-    :return:
-        The imported model.
-    """
-
-    if 'par_map' not in petab_problem.parameter_df:
-        raise ValueError("'par_map' is missing in the parameter table")
-    par_map = get_par_map(petab_problem.parameter_df)
-    if 'parameterScale' not in petab_problem.parameter_df:
-        scale = 'lin'
-    else:
-        scale = petab_problem.parameter_df.parameterScale.to_dict()
-    ss_post_process_callable = None
-    if 'observableFormula' in petab_problem.observable_df:
-        ss_post_process_callable = get_ss_post_process(
-            petab_problem.observable_df)
-    exp_cond_map = get_exp_cond_map(petab_problem.condition_df)
-
-    if len(exp_cond_map) < 2:
-        model = MorpheusModel(model_file=model_dir,
-                              par_map=par_map,
-                              par_scale=scale,
-                              exp_cond_map=exp_cond_map,
-                              ss_post_processing=ss_post_process_callable)
-        return model
-    else:
-        models_list = []
-        models = MorpheusModels(models_list=models_list)
-        for condition, val in exp_cond_map.items():
-            model = MorpheusModel(model_file=model_dir,
-                                  par_map=par_map,
-                                  par_scale=scale,
-                                  exp_cond_map={condition: val},
-                                  ss_post_processing=ss_post_process_callable)
-            models_list.append(model)
-        return models
+# def import_petab_problem(
+#         petab_problem: petab_MS.Problem) -> 'MorpheusModel':
+#     """
+#     Import model from petab problem.
+#
+#     :param petab_problem:
+#         A petab problem containing all relevant information on the model.
+#
+#     :return:
+#         The imported model.
+#     """
+#
+#     if 'par_map' not in petab_problem.parameter_df:
+#         raise ValueError("'par_map' is missing in the parameter table")
+#     par_map = get_par_map(petab_problem.parameter_df)
+#     if 'parameterScale' not in petab_problem.parameter_df:
+#         scale = 'lin'
+#     else:
+#         scale = petab_problem.parameter_df.parameterScale.to_dict()
+#     ss_post_process_callable = None
+#     if 'observableFormula' in petab_problem.observable_df:
+#         ss_post_process_callable = get_ss_post_process(
+#             petab_problem.observable_df)
+#     exp_cond_map = get_exp_cond_map(petab_problem.condition_df)
+#
+#     if len(exp_cond_map) < 2:
+#         model = MorpheusModel(model_file=petab_problem.model_file,
+#                               par_map=par_map,
+#                               par_scale=scale,
+#                               exp_cond_map=exp_cond_map,
+#                               ss_post_processing=ss_post_process_callable)
+#         return model
+#     else:
+#         models_list = []
+#         models = MorpheusModels(models_list=models_list)
+#         for condition, val in exp_cond_map.items():
+#             model = MorpheusModel(model_file=petab_problem.model_file,
+#                                   par_map=par_map,
+#                                   par_scale=scale,
+#                                   exp_cond_map={condition: val},
+#                                   ss_post_processing=ss_post_process_callable)
+#             models_list.append(model)
+#         return models
 
 
 def get_bounds(
