@@ -5,10 +5,10 @@ FitMultiCell
 Fitting of multi-cellular models combining the tools morpheus and pyABC.
 """
 
-from .version import __version__  # noqa: F401
-from .model import *  # noqa: F403, F401
-from .sumstat import *  # noqa: F403, F401
-from .distance import *  # noqa: F403, F401
+from .version import __version__
+from .model import MorpheusModel, MorpheusModels
+from .sumstat import *  # noqa: F403
+from .distance import *  # noqa: F403
 
 import os
 import logging
@@ -21,6 +21,5 @@ except KeyError:
 logger = logging.getLogger("FitMultiCell")
 logger.setLevel(loglevel)
 sh = logging.StreamHandler()
-sh.setFormatter(logging.Formatter(
-    '%(name)s %(levelname)s: %(message)s'))
+sh.setFormatter(logging.Formatter('%(name)s %(levelname)s: %(message)s'))
 logger.addHandler(sh)
