@@ -1,19 +1,23 @@
-import pandas as pd
-import numpy as np
+import copy
+import logging
 import numbers
 import os
-import copy
-import xml.etree.ElementTree as ET
-from typing import List, Callable, Union
-from types import ModuleType
-import logging
 import shutil
+import xml.etree.ElementTree as ET  # noqa: S405
+from types import ModuleType
+from typing import Callable, List, Union
+
+import numpy as np
+import pandas as pd
+
 from ..sumstat import IdSumstatFun
 from ..util import base as util_base
+
 logger = logging.getLogger("FitMultiCell.Model")
+
 try:
-    from pyabc.parameters import Parameter
     from pyabc.external.base import ExternalModel
+    from pyabc.parameters import Parameter
 except ImportError:
     logger.error("pyabc must be installed.")
 
@@ -27,71 +31,72 @@ class MorpheusModel(ExternalModel):
     Parameters
     ----------
 
-    model_file: str
+    model_file:
         The XML file containing the morpheus model.
-    par_map: dict
+    par_map:
         A dictionary from str to str, the keys being the parameter ids
         to be used in pyabc, and the values xpaths in the `morpheus_file`.
-    par_scale: dict or str (default: 'lin')
+    par_scale:
         A dictionary or str to state the scale used to define the parameter
         space, e.g., lin, log10, log2
-    sumstat_funs: List
+    sumstat_funs:
         List of functions to calculate summary statistics. The list entries
         are instances of fitmulticell.sumstat.SumstatFun.
-    executable: str, optional
+    executable:
         The path to the morpheus executable. If None given,
         'morpheus' is used.
-    suffix, prefix: str, optional (default: None, 'morpheus_model_')
+    suffix, prefix:
         Suffix and prefix to use for the temporary folders created.
-    dir: str, optional (default: None)
+    dir:
         Directory to put the temporary folders into. The default is
         the system's temporary files location. Note that these files
         are usually deleted upon system shutdown.
-    show_stdout, show_stderr: bool, optional (default = False, True)
+    show_stdout, show_stderr:
         Whether to show or hide the stdout and stderr streams.
-    raise_on_error: bool, optional (default = False)
+    raise_on_error:
         Whether to raise on an error in the model execution, or
         just continue.
-    name: str, optional (default: None)
+    name:
         A name that can be used to identify the model, as it is
         saved to db. If None is passed, the model_file name is used.
-    time_var: str, optional (default: "time")
+    time_var:
         The name of the time variable as define in Morpheus model.
-    ignore_list: list
+    ignore_list:
         A list of columns to ignore from Morpheus output. This is introduced to
         solve the issue with result that cannot be eliminated from morpheus
         output but yet are not used in the fitting process.
-    timeout: int, optional (default: np.inf)
+    timeout:
         A time in seconds used to early stop Morpheus simulation when exceed.
-    ss_post_processing: Union[Callable, dict], optional
+    ss_post_processing:
         A callable function to perform post processing on Morpheus output. If
         a dict is passed, then specific function will be applied to each
         summary statistics.
-    output_file: str, optional (default: "logger.csv")
+    output_file:
         A name of the file containing the simulation output.
     """
 
-    def __init__(self,
-                 model_file: str,
-                 par_map: dict,
-                 par_scale: Union[dict, str] = "lin",
-                 exp_cond_map: dict = None,
-                 sumstat_funs: List = None,
-                 executable: str = "morpheus",
-                 suffix: str = None,
-                 prefix: str = "morpheus_model_",
-                 dir: str = None,
-                 clean_simulation: bool = False,
-                 show_stdout: bool = False,
-                 show_stderr: bool = True,
-                 raise_on_error: bool = False,
-                 name: str = None,
-                 time_var: str = "time",
-                 ignore_list: list = [],
-                 timeout: int = np.inf,
-                 ss_post_processing: Union[Callable, dict] = None,
-                 output_file: str = "logger.csv"
-                 ):
+    def __init__(
+        self,
+        model_file: str,
+        par_map: dict,
+        par_scale: Union[dict, str] = "lin",
+        exp_cond_map: dict = None,
+        sumstat_funs: List = None,
+        executable: str = "morpheus",
+        suffix: str = None,
+        prefix: str = "morpheus_model_",
+        dir: str = None,
+        clean_simulation: bool = False,
+        show_stdout: bool = False,
+        show_stderr: bool = True,
+        raise_on_error: bool = False,
+        name: str = None,
+        time_var: str = "time",
+        ignore_list: list = None,
+        timeout: float = np.inf,
+        ss_post_processing: Union[Callable, dict] = None,
+        output_file: str = "logger.csv",
+    ):
         if name is None:
             name = model_file
         super().__init__(
@@ -99,12 +104,16 @@ class MorpheusModel(ExternalModel):
             file=model_file,
             fixed_args=None,
             create_folder=True,
-            suffix=suffix, prefix=prefix, dir=dir,
+            suffix=suffix,
+            prefix=prefix,
+            dir=dir,
             clean_simulation=clean_simulation,
             show_stdout=show_stdout,
             show_stderr=show_stderr,
             raise_on_error=raise_on_error,
-            name=name)
+            name=name,
+        )
+
         self.par_map = par_map
         self.par_scale = par_scale
         self.exp_cond_map = exp_cond_map
@@ -112,12 +121,15 @@ class MorpheusModel(ExternalModel):
             if self.exp_cond_map is None:
                 sumstat_funs = [IdSumstatFun()]
             else:
-                sumstat_funs = [IdSumstatFun(
-                    name=list(self.exp_cond_map.keys())[0])]
+                sumstat_funs = [
+                    IdSumstatFun(name=list(self.exp_cond_map.keys())[0])
+                ]
         self.clean_simulation = clean_simulation
         self.sumstat_funs = sumstat_funs
         self._check_sumstat_funs()
         self.time_var = time_var
+        if ignore_list is None:
+            ignore_list = []
         self.ignore_list = ignore_list
         self.timeout = timeout
         self.ss_post_processing = ss_post_processing
@@ -125,9 +137,7 @@ class MorpheusModel(ExternalModel):
         self._sanity_chceck_model()
 
     def __str__(self):
-        s = f"MorpheusModel {{\n" \
-            f"\tname      : {self.name}\n" \
-            f"}}"
+        s = f"MorpheusModel {{\n" f"\tname      : {self.name}\n" f"}}"
         return s
 
     def __repr__(self):
@@ -158,8 +168,10 @@ class MorpheusModel(ExternalModel):
 
         # compute summary statistics
         if status == -15:
-            msg = (f"Simulation exceed time limit limit: {self.timeout}s "
-                   f"for arguments {pars}: returncode {status}.")
+            msg = (
+                f"Simulation exceed time limit limit: {self.timeout}s "
+                f"for arguments {pars}: returncode {status}."
+            )
             logger.warning(msg)
             return -15
         sumstats = self.compute_sumstats(loc)
@@ -192,7 +204,8 @@ class MorpheusModel(ExternalModel):
             return par[0], par[1]
         else:
             raise TypeError(
-                f"par_map[{key}] should be a str or a list/tuple of length 2")
+                f"par_map[{key}] should be a str or a list/tuple of length 2"
+            )
 
     def get_expcondmap_xpath_attr(self, key, attrib='value'):
         """
@@ -214,7 +227,8 @@ class MorpheusModel(ExternalModel):
             return exp_cond[0], exp_cond[1]
         else:
             raise TypeError(
-                f"par_map[{key}] should be a str or a list/tuple of length 2")
+                f"par_map[{key}] should be a str or a list/tuple of length 2"
+            )
 
     def write_modified_model_file(self, file_, pars):
         """
@@ -223,7 +237,7 @@ class MorpheusModel(ExternalModel):
         """
         temp_par = copy.deepcopy(pars)
         rescaled_pars = util_base.scaling_parameter(temp_par, self.par_scale)
-        tree = ET.parse(self.eh.file)
+        tree = ET.parse(self.eh.file)  # noqa: S314
         root = tree.getroot()
         for key, val in rescaled_pars.items():
             xpath, attr = self.get_parmap_xpath_attr(key)
@@ -231,8 +245,7 @@ class MorpheusModel(ExternalModel):
             if node.__len__() == 1:
                 node[0].set(attr, str(val))
             else:
-                raise KeyError(
-                    f"Key {key} is not unique or does not exist.")
+                raise KeyError(f"Key {key} is not unique or does not exist.")
 
         # TODO: better handle the attr for exp_cond_map
         attr = 'value'
@@ -252,12 +265,14 @@ class MorpheusModel(ExternalModel):
                         else:
                             raise KeyError(
                                 f"condition {condition} "
-                                f"is not unique or does not exist.")
+                                f"is not unique or does not exist."
+                            )
             else:
                 raise KeyError(
                     "It seems that the model has more that one condition. "
                     "Please try to use only one condition or"
-                    " use appropriate model class.")
+                    " use appropriate model class."
+                )
 
         tree.write(file_, encoding="utf-8", xml_declaration=True)
 
@@ -269,7 +284,8 @@ class MorpheusModel(ExternalModel):
         sumstat_dict = {'loc': loc}
         if "IdSumstat" in self.sumstat_funs[0].name:
             sumstat_dict = self.sumstat_funs[0](
-                loc, self.ignore_list, self.output_file)
+                loc, self.ignore_list, self.output_file
+            )
             sumstat_dict['loc'] = loc
 
             # here add the prepare function,e.g., 0: val,val,val
@@ -290,7 +306,8 @@ class MorpheusModel(ExternalModel):
         if not len(set(names)) == len(names):
             raise AssertionError(
                 f"The summary statistics passed to MorpheusModel must have"
-                f"unique names, but obtained {names}")
+                f"unique names, but obtained {names}"
+            )
 
     def _sanity_chceck_model(self):
         """
@@ -301,7 +318,7 @@ class MorpheusModel(ExternalModel):
         file_ = os.path.join(loc, "model.xml")
 
         # write new file with for the model
-        tree = ET.parse(self.eh.file)
+        tree = ET.parse(self.eh.file)  # noqa: S314
         tree.write(file_, encoding="utf-8", xml_declaration=True)
 
         # create command
@@ -317,7 +334,8 @@ class MorpheusModel(ExternalModel):
             raise ValueError(
                 f"The model is not running well. "
                 f"The simulation seems to fail. "
-                f"Returncode: {status['returncode']}")
+                f"Returncode: {status['returncode']}"
+            )
         clean_simulation_output(loc)
 
     def call_post_processing_ss(self, sumstats, function_name='main'):
@@ -326,37 +344,44 @@ class MorpheusModel(ExternalModel):
                 sumstats = self.ss_post_processing(sumstats)
             elif isinstance(self.ss_post_processing, dict):
                 if not set(self.ss_post_processing.keys()).issubset(
-                        sumstats.keys()):
+                    sumstats.keys()
+                ):
                     raise ValueError(
                         "the keys on the 'ss_post_processing' does not "
-                        "match the one on the summary statistics names.")
-                if isinstance(list(self.ss_post_processing.values())[0],
-                              Callable):
+                        "match the one on the summary statistics names."
+                    )
+                if isinstance(
+                    list(self.ss_post_processing.values())[0], Callable
+                ):
                     sumstats = self._call_post_processing_ss_use_function(
-                        sumstats)
-                elif isinstance(list(self.ss_post_processing.values())[0],
-                                ModuleType):
+                        sumstats
+                    )
+                elif isinstance(
+                    list(self.ss_post_processing.values())[0], ModuleType
+                ):
                     sumstats = self._call_post_processing_ss_use_module(
-                        sumstats, function_name)
+                        sumstats, function_name
+                    )
             else:
                 raise ValueError(
                     f"the type of 'post_processing_ss' should be str or dict."
-                    f" However, {type(self.ss_post_processing)} was given.")
+                    f" However, {type(self.ss_post_processing)} was given."
+                )
         return sumstats
 
     def _call_post_processing_ss_use_module(self, sumstats, function_name):
         sumstat_pp = {}
-        for key, module in self.ss_post_processing.items():
+        for key, _module in self.ss_post_processing.items():
             try:
-                # sumstat_pp[key] = module.main(sumstats)
                 func = getattr(self.ss_post_processing[key],
                                function_name, None)
-                sumstat_pp[key] = func(sumstats[key])
+                sumstat_pp[key] = func({key: sumstats[key]})
             except Exception as e:
                 raise RuntimeError(
                     f"the selected ss_post_processing function "
                     f"can not be called. Be sure that the main "
-                    f"function called `main(). `{e}")
+                    f"function called `main(). `{e}"
+                )
         return sumstat_pp
 
     def _call_post_processing_ss_use_function(self, sumstats):
@@ -367,7 +392,8 @@ class MorpheusModel(ExternalModel):
             except Exception as e:
                 raise RuntimeError(
                     f"the selected ss_post_processing function "
-                    f"can not be called. `{e}")
+                    f"can not be called. `{e}"
+                )
         return sumstat_pp
 
 
@@ -379,56 +405,51 @@ class MorpheusModels(ExternalModel):
 
     Parameters
     ----------
-    model_file: str
+    model_file:
         The XML file containing the morpheus model.
-    par_map: dict
+    par_map:
         A dictionary from str to str, the keys being the parameter ids
         to be used in pyabc, and the values xpaths in the `morpheus_file`.
-    sumstat_funs: List
+    sumstat_funs:
         List of functions to calculate summary statistics. The list entries
         are instances of fitmulticell.sumstat.SumstatFun.
-    executable: str, optional
+    executable:
         The path to the morpheus executable. If None given,
         'morpheus' is used.
-    suffix, prefix: str, optional (default: None, 'morpheus_model_')
+    suffix, prefix:
         Suffix and prefix to use for the temporary folders created.
-    dir: str, optional (default: None)
+    dir:
         Directory to put the temporary folders into. The default is
         the system's temporary files location. Note that these files
         are usually deleted upon system shutdown.
-    show_stdout, show_stderr: bool, optional (default = False, True)
+    show_stdout, show_stderr:
         Whether to show or hide the stdout and stderr streams.
-    raise_on_error: bool, optional (default = False)
+    raise_on_error:
         Whether to raise on an error in the model execution, or
         just continue.
-    name: str, optional (default: None)
+    name:
         A name that can be used to identify the model, as it is
         saved to db. If None is passed, the model_file name is used.
-    time_var: str, optinal (default: "time")
+    time_var:
         The name of the time variable as define in Morpheus model.
-    ignore_list: list
+    ignore_list:
         A list of columns to ignore from Morpheus output. This is introduced to
         solve the issue with result that cannot be eliminated from morpheus
         output but yet are not used in the fitting process.
-    timeout: int, optional (default: np.inf)
+    timeout:
         A time in seconds used to early stop Morpheus simulation when exceed.
-    ss_post_processing: Union[Callable, dict], optional
+    ss_post_processing:
         A callable function to perform post processing on Morpheus output. If
         a dict is passed, then specific function will be applied to each
         summary statistics.
     """
 
-    def __init__(self,
-                 models_list: List[MorpheusModel],
-                 name: str = None
-                 ):
+    def __init__(self, models_list: List[MorpheusModel], name: str = None):
         self.models_list = models_list
         self.name = name
 
     def __str__(self):
-        s = f"MorpheusModels {{\n" \
-            f"\tname      : {self.name}\n" \
-            f"}}"
+        s = f"MorpheusModels {{\n" f"\tname      : {self.name}\n" f"}}"
         return s
 
     def __repr__(self):
@@ -446,7 +467,7 @@ class MorpheusModels(ExternalModel):
         # a list that will hold result for each experimental condition.
         sumstats_all = {}
         # create target on file system
-        for i, model in enumerate(self.models_list):
+        for model in self.models_list:
             # cond_sumstats = {}
             sumstats = model(pars)
             # for ss_key, ss_val in sumstats.items():
@@ -475,7 +496,8 @@ class MorpheusModels(ExternalModel):
             return par[0], par[1]
         else:
             raise TypeError(
-                f"par_map[{key}] should be a str or a list/tuple of length 2")
+                f"par_map[{key}] should be a str or a list/tuple of length 2"
+            )
 
     def write_modified_models_file(self, file_, pars, exp_cod):
         """
@@ -484,7 +506,7 @@ class MorpheusModels(ExternalModel):
         """
         temp_par = copy.deepcopy(pars)
         rescaled_pars = util_base.scaling_parameter(temp_par, self.par_scale)
-        tree = ET.parse(self.eh.file)
+        tree = ET.parse(self.eh.file)  # noqa: S314
         root = tree.getroot()
         # parameters
         for key, val in rescaled_pars.items():
@@ -493,8 +515,7 @@ class MorpheusModels(ExternalModel):
             if node.__len__() == 1:
                 node[0].set(attr, str(val))
             else:
-                raise KeyError(
-                    f"Key {key} is not unique or does not exist.")
+                raise KeyError(f"Key {key} is not unique or does not exist.")
         # conditions
         for key, val in exp_cod.items():
             xpath, attr = self.get_parmap_xpath_attr(key)
@@ -502,8 +523,7 @@ class MorpheusModels(ExternalModel):
             if node.__len__() == 1:
                 node[0].set(attr, str(val))
             else:
-                raise KeyError(
-                    f"Key {key} is not unique or does not exist.")
+                raise KeyError(f"Key {key} is not unique or does not exist.")
 
         tree.write(file_, encoding="utf-8", xml_declaration=True)
 
@@ -513,37 +533,46 @@ class MorpheusModels(ExternalModel):
                 sumstats = self.ss_post_processing(sumstats)
             elif isinstance(self.ss_post_processing, dict):
                 if not set(self.ss_post_processing.keys()).issubset(
-                        sumstats.keys()):
+                    sumstats.keys()
+                ):
                     raise ValueError(
                         "the keys on the 'ss_post_processing' does not "
-                        "match the one on the summary statistics names.")
-                if isinstance(list(self.ss_post_processing.values())[0],
-                              Callable):
+                        "match the one on the summary statistics names."
+                    )
+                if isinstance(
+                    list(self.ss_post_processing.values())[0], Callable
+                ):
                     sumstats = self._call_post_processing_ss_use_function(
-                        sumstats)
-                elif isinstance(list(self.ss_post_processing.values())[0],
-                                ModuleType):
+                        sumstats
+                    )
+                elif isinstance(
+                    list(self.ss_post_processing.values())[0], ModuleType
+                ):
                     sumstats = self._call_post_processing_ss_use_module(
-                        sumstats, function_name)
+                        sumstats, function_name
+                    )
             else:
                 raise ValueError(
                     f"the type of 'post_processing_ss' should be str or dict."
-                    f" However, {type(self.ss_post_processing)} was given.")
+                    f" However, {type(self.ss_post_processing)} was given."
+                )
         return sumstats
 
     def _call_post_processing_ss_use_module(self, sumstats, function_name):
         sumstat_pp = {}
-        for key, module in self.ss_post_processing.items():
+        for key, _module in self.ss_post_processing.items():
             try:
                 # sumstat_pp[key] = module.main(sumstats)
-                func = getattr(self.ss_post_processing[key],
-                               function_name, None)
+                func = getattr(
+                    self.ss_post_processing[key], function_name, None
+                )
                 sumstat_pp[key] = func(sumstats[key])
             except Exception as e:
                 raise RuntimeError(
                     f"the selected ss_post_processing function "
                     f"can not be called. Be sure that the main "
-                    f"function called `main(). `{e}")
+                    f"function called `main(). `{e}"
+                )
         return sumstat_pp
 
     def _call_post_processing_ss_use_function(self, sumstats):
@@ -554,7 +583,8 @@ class MorpheusModels(ExternalModel):
             except Exception as e:
                 raise RuntimeError(
                     f"the selected ss_post_processing function "
-                    f"can not be called. `{e}")
+                    f"can not be called. `{e}"
+                )
         return sumstat_pp
 
 
@@ -564,19 +594,21 @@ def safe_append_sumstat(sumstat_dict, sumstat, key):
         if key in sumstat_dict:
             raise KeyError(
                 f"Key {key} for sumstat {sumstat} already in the "
-                f"sumstat dict {sumstat_dict}.")
+                f"sumstat dict {sumstat_dict}."
+            )
         sumstat_dict[key] = sumstat
         return
     if isinstance(sumstat, dict):
         if key in sumstat_dict:
             raise KeyError(
                 f"Key {key} for sumstat {sumstat} already in the "
-                f"sumstat dict {sumstat_dict}.")
+                f"sumstat dict {sumstat_dict}."
+            )
         sumstat_dict.update(sumstat)
         return
     raise ValueError(
-        f"Type {type(sumstat)} of sumstat {sumstat} "
-        f"is not permitted.")
+        f"Type {type(sumstat)} of sumstat {sumstat} " f"is not permitted."
+    )
 
 
 def clean_simulation_output(loc):
