@@ -113,8 +113,7 @@ def get_clusters_count(
                 full_list_of_cluster = full_list_of_cluster + \
                     index_list_of_cluster
                 cluster_count = cluster_count + 1
-            morpheus_logger_list_unique = sorted(
-                list(set(morpheus_logger_list)))
+            morpheus_logger_list_unique = sorted(set(morpheus_logger_list))
         if morpheus_logger_list_unique == sorted(cluster_cell_type):
             cluster_count = 1
         cluster_per_time_point[j] = cluster_count
